import json
import logging

from fattmerchant.exceptions import InvalidRequestDataException
from fattmerchant.models import Transaction

logger = logging.getLogger(__name__)


class ChargesController(object):
    """
    Class to allow interfacing with charges within the Fattmerchant API
    """
    def __init__(self, request):
        self.request = request

    def create(self, id=None, meta=None, total=None, pre_auth=None):
        """
        Creates a charge for a merchant through the API

        :param id: A payment method ID
        :type id: string
        :param meta: Any meta data for the payment *i.e. tax*
        :type meta: dict
        :param total: The total amount to charge
        :type total: float
        :param pre_auth: Whether or not to confirm funds exist
        :type pre_auth: boolean

        :return: A single transaction object
        :rtype: :doc:`../models/transaction`

        :raise InvalidRequestDataException: Raised if **id** is not
            provided or **meta** is not provided or **total** is not provided

        """

        if not isinstance(id, (str, unicode)) or id is None:
            msg = "An id of type string is required to complete the request."

            raise InvalidRequestDataException(msg)

        if not isinstance(meta, dict) or total is None:
            msg = "A total of type int or float is required to complete " \
                "the request."

            raise InvalidRequestDataException(msg)

        if not isinstance(total, (int, float)) or total is None:
            msg = "A meta object of type dict is required to complete " \
                "the request."

            raise InvalidRequestDataException(msg)

        endpoint = "charge"

        payload = {"payment_method_id": id, "meta": meta, "total": total}

        if pre_auth is not None:
            payload["pre_auth"] = pre_auth

        response = json.loads(
            self.request.post(endpoint=endpoint, payload=payload)
        )

        return Transaction(response)



from __future__ import absolute_import
__author__ = "tanmay.datta86@gmail.com"

import logging
import json

from fattmerchant.exceptions import InvalidRequestDataException
from fattmerchant.models import Customer, PaymentMethod

logger = logging.getLogger(__name__)


class CustomersController(object):
    """
    Class to interface with the customer resource in the Core API
    """
    def __init__(self, request):
        self.request = request

    def create(self, payload):
        """
        Creates a new customer for a merchant through the API

        :param payload: A dictionary with all data necessary to create a customer

            .. note:: **firstname**, **lastname**, **company**, and **email** are required
            .. code-block:: json

                {
                    "firstname": "John",
                    "lastname": "Smith",
                    "company": "ABC INC",
                    "email": "daniel@fattmerchant.com",
                    "files": [],
                    "phone": "1234567898",
                    "merchant_id": "12345678",
                    "address_1": "123 Rite Way",
                    "address_2": "Unit 12",
                    "address_city": "Orlando",
                    "address_state": "FL",
                    "address_zip": "32801",
                    "address_country": "USA",
                    "reference": "",
                    "cc_emails": [],
                    "cc_sms": [],
                    "notes": "",
                    "options": {},
                    "allow_invoice_credit_card_payments": true
                }

        :type payload: dict

        :return: A single customer object
        :rtype: :doc:`../models/customer`

        :raise InvalidRequestDataException: Raised if **firstname**,
            **lastname**, **company**, and **email** are not provided

        """  # NOQA: E501

        endpoint = u'customer'
        required_fields = {"firstname", "lastname", "company", "email"}

        if not all(
            field in payload and isinstance(payload[field], (str, unicode))
            for field in required_fields
        ):
            msg = "A dict with at least a firstname, lastname, email, and " \
                "company is required to perform this request."

            raise InvalidRequestDataException(msg)

        response = json.loads(
            self.request.post(endpoint=endpoint, payload=payload)
        )

        return Customer(response)

    def list(self):
        """
        Gets a list of customers from the API

        :param: None

        :return: A list of customer objects
        :rtype: :doc:`../models/customer`

        """
        endpoint = "customer"

        response = json.loads(self.request.get(endpoint=endpoint))

        customers = []

        for customer_data in response["data"]:
            customers.append(Customer(customer_data))

        return customers

    def get(self, id=None):
        """
        Gets a single customer's details from the API

        :param id: A customer ID
        :type id: string

        :return: A single customer object
        :rtype: :doc:`../models/customer`

        :raise InvalidRequestDataException: Raised if **id** is not provided

        """

        if not isinstance(id, (str, unicode)) or id is None:
            msg = "An id of type string has to be passed in with the request."

            raise InvalidRequestDataException(msg)

        endpoint = "customer/{}".format(id)
        response = self.request.get(endpoint=endpoint)

        return Customer(json.loads(response))

    def payment_methods(self, id=None):
        """
        Gets a list of payment methods for a customer from the API

        :param id: A customer's ID
        :type id: string

        :return: A list of payment method objects
        :rtype: :doc:`../models/payment_method`

        :raise InvalidRequestDataException: Raised if **id** is not provided

        """

        if not isinstance(id, (str, unicode)) or id is None:
            msg = "An id of type string has to be passed in with the request."

            raise InvalidRequestDataException(msg)

        endpoint = "customer/{}/payment-method".format(id)

        response = json.loads(self.request.get(endpoint=endpoint))

        payment_methods = []

        for payment_method_data in response:
            payment_methods.append(PaymentMethod(payment_method_data))

        return payment_methods



from datetime import datetime


class Merchant(object):
    """
    Merchant model class
    """ 
    def __init__(self, data):
        self.id = data.get("id")
        """ Id of the merchant """

        self.mid = data.get("mid")
        """ Id assigned to merchant via gateway """

        self.status = data.get("status")
        """ Status of the merchant """

        self.subdomain = data.get("subdomain")
        """ Subdomain of the merchant """

        self.company_name = data.get("company_name")
        """ Company name of the merchant """

        self.display_name = data.get("display_name")
        """ Display name of the merchant """

        self.contact_name = data.get("contact_name")
        """ Contact name of the merchant """

        self.contact_email = data.get("contact_email")
        """ Contact email of the merchant """

        self.contact_phone = data.get("contact_phone")
        """ Contact phone of the merchant """

        self.address_1 = data.get("address_1")
        """ Address of the merchant """

        self.address_2 = data.get("address_2")
        """ Address additional details of the merchant """

        self.address_city = data.get("address_city")
        """ City for the address of the merchant """

        self.address_state = data.get("address_state")
        """ State for the address of the merchant """

        self.address_zip = data.get("address_zip")
        """ Zip code for the address of the merchant """

        self.hosted_payments_token = data.get("hosted_payments_token")
        """ Country for the address of the merchant """

        self.options = data.get("options")
        """ Optional metadata in JSON format about the merchant """

        self.notes = data.get("notes")
        """ Any notes about the merchant """

        self.gateway_type = data.get("gateway_type")
        """ Gateway type of the merchant (deprecated) """

        self.vendor_keys = data.get("vendor_keys")
        """ Vendor keys for the merchant """

        self.processor = data.get("processor")
        """ Payment processor of the merchant """

        self.partner = data.get("partner")
        """ Optional tracking field of where the merchant came from """

        self.product_type = data.get("product_type")
        """ Tracking field for which product the merchant is using """

        self.is_enterprise = data.get("is_enterprise")
        """ Whether the merchant is an enterprise merchant or not """

        self.is_payfac = data.get("is_payfac")
        """ Whether the merchant is a payfac merchant """

        self.fm_billing_schedule_id = data.get("fm_billing_schedule_id")
        """ Schedule id for when the merchant is to be billed """

        self.welcome_email_sent_at = datetime.strptime(
            data.get("welcome_email_sent_at"), '%Y-%m-%d %H:%M:%S'
        ) if data.get("welcome_email_sent_at") else None
        """ When the welcome email was sent """

        self.created_at = datetime.strptime(
            data.get("created_at"), '%Y-%m-%d %H:%M:%S'
        ) if data.get("created_at") else None
        """ When the customer was created """

        self.updated_at = datetime.strptime(
            data.get("updated_at"), '%Y-%m-%d %H:%M:%S'
        ) if data.get("updated_at") else None
        """ The last time the customer was updated """

        self.deleted_at = datetime.strptime(
            data.get("deleted_at"), '%Y-%m-%d %H:%M:%S'
        ) if data.get("deleted_at") else None
        """ When the customer was deleted """

        self.brand = data.get("brand")
        """ Brand of the merchant """

        self.branding = data.get("branding")
        """ The logo for the merchant """

        self.allow_ach = data.get("allow_ach")
        """ Whether to allow ACH transactions """

        self.is_portal = data.get("is_portal")
        """ Whether the merchant is a portal merchant or not """

        self.allow_credits = data.get("allow_credits")
        """ Whether credits are allowed for this merchant or not """

        self.allow_terminal = data.get("allow_terminal")
        """ Whether the merchant is able to process terminal transactions """
    def __repr__(self):
        repr = '{}(' \
            'id: {!r}, ' \
            'mid: {!r}, ' \
            'status: {!r}, ' \
            'subdomain: {!r}, ' \
            'company_name: {!r}, ' \
            'display_name: {!r}, ' \
            'contact_name: {!r}, ' \
            'contact_email: {!r}, ' \
            'contact_phone: {!r}, ' \
            'address_1: {!r}, ' \
            'address_2: {!r}, ' \
            'address_city: {!r}, ' \
            'address_state: {!r}, ' \
            'address_zip: {!r}, ' \
            'hosted_payments_token: {!r}, ' \
            'options: {!r}, ' \
            'notes: {!r}, ' \
            'gateway_type: {!r}, ' \
            'vendor_keys: {!r}, ' \
            'processor: {!r}, ' \
            'partner: {!r}, ' \
            'product_type: {!r}, ' \
            'is_enterprise: {!r}, ' \
            'is_payfac: {!r}, ' \
            'fm_billing_schedule_id: {!r}, ' \
            'welcome_email_sent_at: {!r}, ' \
            'created_at: {!r}, ' \
            'updated_at: {!r}, ' \
            'deleted_at: {!r}, ' \
            'brand: {!r}, ' \
            'branding: {!r}, ' \
            'allow_ach: {!r}, ' \
            'is_portal: {!r}, ' \
            'allow_credits: {!r}, ' \
            'allow_terminal: {!r})'.format(
                self.__class__.__name__,
                self.id,
                self.mid,
                self.status,
                self.subdomain,
                self.company_name,
                self.display_name,
                self.contact_name,
                self.contact_email,
                self.contact_phone,
                self.address_1,
                self.address_2,
                self.address_city,
                self.address_state,
                self.address_zip,
                self.hosted_payments_token,
                self.options,
                self.notes,
                self.gateway_type,
                self.vendor_keys,
                self.processor,
                self.partner,
                self.product_type,
                self.is_enterprise,
                self.is_payfac,
                self.fm_billing_schedule_id,
                self.welcome_email_sent_at,
                self.created_at,
                self.updated_at,
                self.deleted_at,
                self.brand,
                self.branding,
                self.allow_ach,
                self.is_portal,
                self.allow_credits,
                self.allow_terminal
            )

        return repr



from datetime import datetime

from fattmerchant.models import Customer


class PaymentMethod(object):
    """
    Payment Method model class
    """
    def __init__(self, data):
        self.id = data.get("id")
        """ Id of the payment method """

        self.customer_id = data.get("customer_id")
        """ Id of customer the payment method belongs to """

        self.merchant_id = data.get("merchant_id")
        """ Id of merchant the payment method belongs to """

        self.user_id = data.get("user_id")
        """ Id of user the payment method belongs to """

        self.nickname = data.get("nickname")
        """ Nickname of the payment method """

        self.has_cvv = data.get("has_cvv")
        """ Whether the payment method has a cvv or not """

        self.is_default = data.get("is_default")
        """ Whether the payment method is the default or not """

        self.method = data.get("method")
        """ The type of payment method """

        self.person_name = data.get("person_name")
        """ The name on a credit card payment method """

        self.card_type = data.get("card_type")
        """ The type of a credit card payment method """

        self.card_last_four = data.get("card_last_four")
        """ The last four numbers on a credit card payment method """

        self.card_exp = data.get("card_exp")
        """ The expiration date on a credit card payment method """

        self.bank_name = data.get("bank_name")
        """ The bank name on an ACH payment method """

        self.bank_type = data.get("bank_type")
        """ The bank type on an ACH payment method """

        self.bank_holder_type = data.get("bank_holder_type")
        """ The bank holder type on an ACH payment method """

        self.address_1 = data.get("address_1")
        """ The address for the payment method """

        self.address_2 = data.get("address_2")
        """ The address additional details for the payment method """

        self.address_city = data.get("address_city")
        """ The address city for the payment method """

        self.address_state = data.get("address_state")
        """ The address state for the payment method """

        self.address_zip = data.get("address_zip")
        """ The address zip for the payment method """

        self.address_country = data.get("address_country")
        """ The address country for the payment method """

        self.purged_at = datetime.strptime(
            data.get("purged_at"), '%Y-%m-%d %H:%M:%S'
        ) if data.get("purged_at") else None
        """ When the payment method was purged """

        self.created_at = datetime.strptime(
            data.get("created_at"), '%Y-%m-%d %H:%M:%S'
        ) if data.get("created_at") else None
        """ When the payment method was created """

        self.updated_at = datetime.strptime(
            data.get("updated_at"), '%Y-%m-%d %H:%M:%S'
        ) if data.get("updated_at") else None
        """ When the payment method was updated """

        self.deleted_at = datetime.strptime(
            data.get("deleted_at"), '%Y-%m-%d %H:%M:%S'
        ) if data.get("deleted_at") else None
        """ When the payment method was deleted """

        self.meta = data.get("meta")
        """ Any meta data for the payment method """

        self.bin_type = data.get("bin_type")
        """ The bank name on an ACH payment method """

        self.card_exp_datetime = datetime.strptime(
            data.get("card_exp_datetime"), '%Y-%m-%d %H:%M:%S'
        ) if data.get("card_exp_datetime") else None
        """ The the expiry date/time of the credit card """

        self.is_usable_in_vt = data.get("is_usable_in_vt")
        """ Whether or not the payment method is usable in Omni """

        self.customer = Customer(data.get("customer")) \
            if data.get("customer") else None
        """ The customer for the payment method """
    def __repr__(self):
        repr = '{}(' \
            'id: {!r}, ' \
            'customer_id: {!r}, ' \
            'merchant_id: {!r}, ' \
            'user_id: {!r}, ' \
            'nickname: {!r}, ' \
            'has_cvv: {!r}, ' \
            'is_default: {!r}, ' \
            'method: {!r}, ' \
            'person_name: {!r}, ' \
            'card_type: {!r}, ' \
            'card_last_four: {!r}, ' \
            'card_exp: {!r}, ' \
            'bank_name: {!r}, ' \
            'bank_type: {!r}, ' \
            'bank_holder_type: {!r}, ' \
            'address_1: {!r}, ' \
            'address_2: {!r}, ' \
            'address_city: {!r}, ' \
            'address_state: {!r}, ' \
            'address_zip: {!r}, ' \
            'address_country: {!r}, ' \
            'purged_at: {!r}, ' \
            'deleted_at: {!r}, ' \
            'created_at: {!r}, ' \
            'updated_at: {!r}, ' \
            'meta: {!r}, ' \
            'bin_type: {!r}, ' \
            'card_exp_datetime: {!r}, ' \
            'is_usable_in_vt: {!r}, ' \
            'customer: {!r})'.format(
                self.__class__.__name__,
                self.id,
                self.customer_id,
                self.merchant_id,
                self.user_id,
                self.nickname,
                self.has_cvv,
                self.is_default,
                self.method,
                self.person_name,
                self.card_type,
                self.card_last_four,
                self.card_exp,
                self.bank_name,
                self.bank_type,
                self.bank_holder_type,
                self.address_1,
                self.address_2,
                self.address_city,
                self.address_state,
                self.address_zip,
                self.address_country,
                self.purged_at,
                self.deleted_at,
                self.created_at,
                self.updated_at,
                self.meta,
                self.bin_type,
                self.card_exp_datetime,
                self.is_usable_in_vt,
                self.customer,
            )

        return repr



from fattmerchant.request import Request
from fattmerchant.controllers import TransactionsController, \
    CustomersController, ChargesController, TeamsController, \
    DepositsController


class FMClient(object):
    """
    Class to set up the Fattmerchant Python SDK
    """
    def __init__(self, api_key, env="prod"):
        request = Request(api_key, env)

        self.customers = CustomersController(request)
        self.teams = TeamsController(request)
        self.transactions = TransactionsController(request)
        self.deposits = DepositsController(request)
        self.charges = ChargesController(request)



import logging

logger = logging.getLogger(__name__)


class FattmerchantException(Exception):
    """
    Base exception class for the Fattmerchant Client

    :param str message: Message to use for this exception.
    """
    default_code = 422
    default_message = 'An error has occured.'

    def __init__(self, status_code=None, message=None):
        self._message = message or self.default_message
        self.code = status_code or self.default_code

        super(Exception, self).__init__(self._message)


class InvalidTokenException(FattmerchantException):
    """
    Exception that should be raised when the token fails to validate
    for any reason.

    :param str message: Message to use for this exception.
    """
    default_code = 401
    default_message = 'The token used is invalid.'


class ResourceDoesNotExistException(FattmerchantException):
    """
    Exception that should be raised when a resource is expected but none
    are returned

    :param str message: Message to use for this exception.
    """
    default_code = 404
    default_message = 'The resource requested does not exist.'


class ResourceNotCreatedException(FattmerchantException):
    """
    Exception that should be raised when a resource is could not be
    created.

    :param str message: Message to use for this exception.
    """
    default_code = 422
    default_message = 'The resource could not be created.'


class DuplicateResourceException(FattmerchantException):
    """
    Exception that should be raised when an attempt to create or update a
    resource would have resulted in a duplicate.

    :param str message: Message to use for this exception.
    """
    default_code = 409
    default_message = 'The resource trying to be created already exists.'


class InvalidRequestDataException(FattmerchantException):
    """
    Exception that should be raised if any expected data for a request
    is missing or invalid.

    :param str message: Message to use for this exception.
    """
    default_code = 422
    default_message = 'The request data is missing or invalid.'



from fattmerchant.controllers.transactions import TransactionsController    # NOQA: F401, E501
from fattmerchant.controllers.customers import CustomersController    # NOQA: F401, E501
from fattmerchant.controllers.charges import ChargesController    # NOQA: F401, E501
from fattmerchant.controllers.teams import TeamsController    # NOQA: F401, E501
from fattmerchant.controllers.deposits import DepositsController    # NOQA: F401, E501



from fattmerchant.models.customer import Customer    # noqa: F401, E501
from fattmerchant.models.team import Team    # noqa: F401, E501
from fattmerchant.models.deposit import DepositBatch, DepositDetails    # noqa: F401, E501
from fattmerchant.models.payment_method import PaymentMethod    # noqa: F401, E501
from fattmerchant.models.user import User    # noqa: F401, E501
from fattmerchant.models.merchant import Merchant    # noqa: F401, E501
from fattmerchant.models.transaction import Transaction    # noqa: F401, E501



from __future__ import absolute_import

import logging.config

from os import path, getenv
from json import load


def setup_logging(
    default_path='logging.json', default_level=logging.INFO, env_key='LOG_CFG'
):
    """
    Setup logging configuration
    """
    log_path = default_path
    value = getenv(env_key, None)

    if value:
        log_path = value

    log_path = path.join(path.dirname(path.abspath(__file__)), log_path)

    if path.exists(log_path):
        with open(log_path, 'rt') as f:
            config = load(f)
        logging.config.dictConfig(config)
    else:
        logging.basicConfig(level=default_level)


setup_logging()



import requests
import os
import logging
import json

from fattmerchant.exceptions import FattmerchantException, \
    InvalidTokenException, ResourceDoesNotExistException, \
    DuplicateResourceException, InvalidRequestDataException

self_path = os.path.dirname(__file__)
logger = logging.getLogger(__name__)


class Request(object):
    """
    Class to get a valid fattmerchant request format

    """
    def __init__(self, api_key, env="prod"):
        self.api_key = api_key
        self.use_env(env)

    def use_env(self, env):
        """
        change enviornment for testing

        :param env: string of enviornment can be "local", "demo" or "prod"

        """

        prod_url = 'https://apiprod.fattlabs.com'
        prod_fq_url = 'https://apiprod.fattlabs.com/query'
        demo_url = 'https://apidemo.fattlabs.com'
        demo_fq_url = 'https://apidemo.fattlabs.com/query'
        local_url = 'http://localhost:8000'
        local_fq_url = 'http://localhost:3005'

        self.env = env
        self.api = {}

        if (env == "prod"):
            self.api["core"] = prod_url
            self.api["fq"] = prod_fq_url
        elif (env == "demo"):
            self.api["core"] = demo_url
            self.api["fq"] = demo_fq_url
        elif (env == "local"):
            self.api["core"] = local_url
            self.api["fq"] = local_fq_url

    def __make_request(
        self, api_type, method, endpoint, options={}, payload=None
    ):
        url = "{api}/{endpoint}{query_string}".format(
            api=self.api[api_type],
            endpoint=endpoint,
            query_string=self.__build_query_string(options)
        )

        headers = self.__build_headers()

        # Get the correct "requests" class method by passing
        # in the request method that we want
        req = getattr(requests, method, 'get')

        if payload is not None:
            res = req(url, headers=headers, data=json.dumps(payload))
        else:
            res = req(url, headers=headers)

        if res.status_code == 200:
            return res.text

        msg = "ERROR occured while trying to send a {} request: " \
            "URL: {}, HEADERS: {}, PAYLOAD: {} " \
            "STATUS CODE: {}, RESPONSE: {}"

        logger.error(
            msg.format(
                method.upper(), url, headers, payload, res.status_code,
                res.text
            )
        )

        if res.status_code == 401:
            raise InvalidTokenException(message=res.text)
        elif res.status_code == 404:
            raise ResourceDoesNotExistException(message=res.text)
        elif res.status_code == 409:
            raise DuplicateResourceException(message=res.text)
        elif res.status_code == 400:
            raise InvalidRequestDataException(
                status_code=400, message=res.text
            )
        elif res.status_code == 422:
            raise InvalidRequestDataException(message=res.text)
        else:
            raise FattmerchantException(message=res.text)

    def __build_headers(self):
        return {
            'Content-Type': 'application/json',
            'Authorization': 'Bearer {api_key}'.format(api_key=self.api_key),
            'Accept': 'application/json'
        }

    def __build_query_string(self, params):
        if (len(params) == 0):
            return ""

        query_strings = []

        for key, value in params.items():
            query_strings.append("{}={}".format(key, value))

        return "?{}".format("&".join(query_strings))

    def get(self, api='core', endpoint='', options={}):
        """
        Calls respective api with a `GET` HTTP method

        :param str api: Fattmerchant API base url to be requested
        :param str endpoint: API endpoint to be requested
        """
        return self.__make_request(api, 'get', endpoint, options=options)

    def post(self, api='core', endpoint='', payload={}):
        """
        Calls respective api with a `POST` HTTP method

        :param str api: Fattmerchant API base url to be requested
        :param str endpoint: API endpoint to be requested
        :param dict payload: Data to be sent along with API request
        """
        return self.__make_request(api, 'post', endpoint, payload=payload)

    def put(self, api='core', endpoint='', payload={}):
        """
        Calls respective api with a `PUT` HTTP method

        :param str api: Fattmerchant API base url to be requested
        :param str endpoint: API endpoint to be requested
        :param dict payload: Data to be sent along with API request
        """
        return self.__make_request(api, 'put', endpoint, payload=payload)



from datetime import datetime


class DepositBatch(object):
    """
    Deposit batch model class
    """
    def __init__(self, data):
        self.batch_id = data.get("batch_id")
        """ Id of the deposit batch """

        self.batched_at = datetime.strptime(
            data.get("batched_at"), '%Y-%m-%d'
        ) if data.get("batched_at") else None
        """ When the deposit batch was created """

        self.last_transaction = data.get("last_transaction")
        """ The last transaction of the deposit batch """

        self.count = data.get("count")
        """ The number of deposits in the batch """

        self.sum = data.get("sum")
        """ The sum of the totals from the deposits in the batch """

        self.avg = data.get("avg")
        """ The avg of the totals from the deposits in the batch """

        self.min = data.get("min")
        """ The minimum total of the totals from the deposits in the batch """

        self.max = data.get("max")
        """ The maximum total of the totals from the deposits in the batch """

        self.std = data.get("std")
        """
        The standard deviation of the totals from the deposits in the batch
        """

        self.fees = data.get("fees")
        """
        The total amount of money the merchant is paying in fees for all
        deposits
        """
    def __repr__(self):
        repr = '{}(' \
            'batch_id: {!r}, ' \
            'batched_at: {!r}, ' \
            'last_transaction: {!r}, ' \
            'count: {!r}, ' \
            'sum: {!r}, ' \
            'avg:{!r}, '\
            'min: {!r}, ' \
            'max: {!r}, ' \
            'std: {!r}, ' \
            'fees: {!r})'.format(
                self.__class__.__name__,
                self.batch_id,
                self.batched_at,
                self.last_transaction,
                self.count,
                self.sum,
                self.avg,
                self.min,
                self.max,
                self.std,
                self.fees
            )

        return repr


class DepositDetails(object):
    """
    Deposit details model class
    """
    def __init__(self, data):
        self.batch_id = data.get("batch_id", None)
        """ Id of the deposit batch """

        self.batched_at = datetime.strptime(
            data.get("batched_at"), '%Y-%m-%d %H:%M:%S'
        ) if data.get("batched_at") else None
        """ When the deposit batch was created """

        self.auth_id = data.get("auth_id", None)
        """ The auth id of the deposit """

        self.created_at = datetime.strptime(
            data.get("created_at"), '%Y-%m-%d %H:%M:%S'
        ) if data.get("created_at") else None
        """ When the deposit was created """

        self.total = data.get("total", None)
        """ The total amount of the deposit """

        self.last_four = data.get("last_four", None)
        """
        The last four numbers of the credit card used for the deposit
        """

        self.card_type = data.get("card_type", None)
        """ The type of credit card used for the deposit """
    def __repr__(self):
        repr = '{}(' \
            'batch_id: {!r}, ' \
            'batched_at: {!r}, ' \
            'auth_id: {!r}, ' \
            'created_at: {!r}, ' \
            'total: {!r}, ' \
            'last_four: {!r}, ' \
            'card_type: {!r})'.format(
                self.__class__.__name__,
                self.batched_at,
                self.batch_id,
                self.auth_id,
                self.created_at,
                self.total,
                self.last_four,
                self.card_type,
            )

        return repr



from datetime import datetime


class Customer(object):
    """
    Customer model class
    """
    def __init__(self, data):
        self.id = data.get("id")
        """ Id of the customer """

        self.firstname = data.get("firstname")
        """ First name of the customer """

        self.lastname = data.get("lastname")
        """ Last name of the customer """

        self.company = data.get("company")
        """ Company of the customer """

        self.email = data.get("email")
        """ Email of the customer """

        self.cc_emails = data.get("cc_emails")
        """
        Extra email addresses all emails will also be sent to
        for customer receipts or invoices
        """

        self.cc_sms = data.get("cc_sms")
        """
        Extra phone numbers all texts will also be sent to
        for customer receipts or invoices
        """

        self.phone = data.get("phone")
        """ Phone number of the customer """

        self.address_1 = data.get("address_1")
        """ Address of the customer """

        self.address_2 = data.get("address_2")
        """ Address additional details of the customer """

        self.address_city = data.get("address_city")
        """ City for the address of the customer """

        self.address_state = data.get("address_state")
        """ State for the address of the customer """

        self.address_zip = data.get("address_zip")
        """ Zip code for the address of the customer """

        self.address_country = data.get("address_country")
        """ Country for the address of the customer """

        self.notes = data.get("notes")
        """ Any notes about the customer """

        self.reference = data.get("reference")
        """ Customizable name to reference customer """

        self.options = data.get("options")
        """ Optional metadata in JSON format about the customer """

        self.created_at = datetime.strptime(
            data.get("created_at"), '%Y-%m-%d %H:%M:%S'
        ) if data.get("created_at") else None
        """ When the customer was created """

        self.updated_at = datetime.strptime(
            data.get("updated_at"), '%Y-%m-%d %H:%M:%S'
        ) if data.get("updated_at") else None
        """ When the customer was updated """

        self.deleted_at = datetime.strptime(
            data.get("deleted_at"), '%Y-%m-%d %H:%M:%S'
        ) if data.get("deleted_at") else None
        """ When the customer was deleted """

        self.allow_invoice_credit_card_payments = data.get(
            "allow_invoice_credit_card_payments"
        )
        """ Whether to allow credit card payments for customer on invoices """

        self.gravatar = data.get("gravatar")
        """ Gravatar of the customer """
    def __repr__(self):
        repr = '{}(' \
            'id: {!r}, ' \
            'firstname: {!r}, ' \
            'lastname: {!r}, ' \
            'company: {!r}, ' \
            'email: {!r}, ' \
            'cc_emails: {!r}, ' \
            'cc_sms: {!r}, ' \
            'phone: {!r}, ' \
            'address_1: {!r}, ' \
            'address_2: {!r}, ' \
            'address_city: {!r}, ' \
            'address_state: {!r}, ' \
            'address_zip: {!r}, ' \
            'address_country: {!r}, ' \
            'notes: {!r}, ' \
            'reference: {!r}, ' \
            'options: {!r}, ' \
            'created_at: {!r}, ' \
            'updated_at: {!r}, ' \
            'deleted_at: {!r}, ' \
            'allow_invoice_credit_card_payments: {!r}, ' \
            'gravatar: {!r})'.format(
                self.__class__.__name__,
                self.id,
                self.firstname,
                self.lastname,
                self.company,
                self.email,
                self.cc_emails,
                self.cc_sms,
                self.phone,
                self.address_1,
                self.address_2,
                self.address_city,
                self.address_state,
                self.address_zip,
                self.address_country,
                self.notes,
                self.reference,
                self.options,
                self.created_at,
                self.updated_at,
                self.deleted_at,
                self.allow_invoice_credit_card_payments,
                self.gravatar
            )

        return repr



import json
import logging

from fattmerchant.exceptions import InvalidRequestDataException
from fattmerchant.models import Transaction

logger = logging.getLogger(__name__)


class TransactionsController(object):
    """
    Class interface with transactions within the Fattmerchant API
    """
    def __init__(self, request):
        self.request = request

    def list(self, options={}):
        """
        Gets a list of transactions from the API

        :param options: A dictionary with any optional query params

            .. code-block:: json

                {
                    "per_page": 10,
                    "page": 1
                }

        :type options: dict

        :return: A list of transaction objects
        :rtype: :doc:`../models/transaction`

        """

        endpoint = "transaction"
        response = json.loads(
            self.request.get(endpoint=endpoint, options=options)
        )

        transactions = []

        for transaction_data in response["data"]:
            transactions.append(Transaction(transaction_data))

        return transactions

    def get(self, id=None):
        """
        Gets a single transaction's details from the API

        :param id: A transaction ID
        :type id: string

        :return: A single transaction object
        :rtype: :doc:`../models/transaction`

        :raise InvalidRequestDataException: Raised if **id** is not provided

        """

        if not isinstance(id, (str, unicode)) or id is None:
            msg = "An id of type string has to be passed in with the request."

            raise InvalidRequestDataException(msg)

        endpoint = "transaction/{}".format(id)
        response = self.request.get(endpoint=endpoint)

        return Transaction(json.loads(response))

    def refund(self, id=None, amount=None):
        """
        Refund a certain amount of a transaction

        :param id: A transaction ID
        :type id: string
        :param amount: The amount to be refunded
        :type amount: int|float

        :return: A single transaction object
        :rtype: :doc:`../models/transaction`

        :raise InvalidRequestDataException: Raised if **id** is not provided
            and if **amount** is not provided

        """

        if not isinstance(id, (str, unicode)) or id is None:
            msg = "An id of type string has to be passed in with the request."

            raise InvalidRequestDataException(msg)

        if not isinstance(amount, (int, float)) or amount is None:
            msg = "An amount of type int or float has to be passed " \
                "in with the request."

            raise InvalidRequestDataException(msg)

        endpoint = "transaction/{}/refund".format(id)
        payload = {"total": amount}

        response = self.request.post(endpoint=endpoint, payload=payload)

        return Transaction(json.loads(response))



def compare_json_data(source_data_a, source_data_b):
    def compare(data_a, data_b):
        # type: list
        if (type(data_a) is list):
            # is [data_b] a list and of same length as [data_a]?
            if ((type(data_b) != list) or (len(data_a) != len(data_b))):
                return False

            # iterate over list items
            for list_index, list_item in enumerate(data_a):
                # compare [data_a] list item against [data_b] at index
                if (not compare(list_item, data_b[list_index])):
                    return False

            # list identical
            return True

        if (type(data_a) is dict):
            # is [data_b] a dictionary?
            if (type(data_b) != dict):
                return False

            # iterate over dictionary keys
            for dict_key, dict_value in data_a.items():
                # key exists in [data_b] dictionary, and same value?
                if (
                    (dict_key not in data_b)
                    or (not compare(dict_value, data_b[dict_key]))
                ):
                    return False

            # dictionary identical
            return True

        # simple value - compare both value and type for equality
        return ((data_a == data_b) and (type(data_a) is type(data_b)))

    # compare a to b, then b to a
    return (
        compare(source_data_a, source_data_b)
        and compare(source_data_b, source_data_a)
    )



import json
import logging

from fattmerchant.exceptions import InvalidRequestDataException, \
    ResourceNotCreatedException
from fattmerchant.models import Team

logger = logging.getLogger(__name__)


class TeamsController(object):
    """
    Class to interface with the team resource in the Core API
    """
    def __init__(self, request):
        self.request = request

    def _create_team(self, payload):
        """
        Private method to make the API call to create a new team

        Args:
            payload (dict): A dictionary with all data necessary to create a
                team

        Returns:
            tuple: A tuple with an api key, a user for a merchant, and said
                merchant

        Raises:
            ResourceNotCreatedException: If the team couldn't be accessed
                then it wasn't able to be created

        """

        endpoint = "team"
        response = json.loads(
            self.request.post(endpoint=endpoint, payload=payload)
        )

        try:
            latestTeam = response[0]

            for team in response:
                if (
                    team["merchant"]["created_at"] >
                    latestTeam["merchant"]["created_at"]
                ):
                    latestTeam = team

            return (
                latestTeam["token"], latestTeam["user"], latestTeam["merchant"]
            )
        except IndexError:
            logger.error("The team was not able to be created")

            raise ResourceNotCreatedException

    def create(self, payload):
        """
        Creates a new team through the API

        :param payload: A dictionary with all data necessary to create a team

            .. note:: **company_name** and **company_email** are required
            .. code-block:: JSON

                {
                    "company_name": "Fattmerchant",
                    "comtact_name": "Fattmerchant",
                    "contact_email": "info2@fattmerchant.com",
                    "contact_phone": "8555503288",
                    "display_name": "Fattmerchant",
                    "address_1": "25 Wall Street",
                    "address_2": "Suite 1",
                    "address_city": "Orlando",
                    "address_state": "FL",
                    "address_zip": "32801",
                    "options": {}
                }

        :type payload: dict

        :return: A tuple with a team object and said team's api key
        :rtype: (:doc:`../models/team`, string)

        :raise InvalidRequestDataException: Raised if **company_name** is not
            provided or **contact_email** is not provided

        :raise ResourceNotCreatedException: Raised if the team could not be
            created within the API

        """

        if "company_name" not in payload or not isinstance(
            payload["company_name"], (str, unicode)
        ):
            msg = "A company name of type string is required to complete the \
                request."

            raise InvalidRequestDataException(msg)

        if "contact_email" not in payload or not isinstance(
            payload["contact_email"], (str, unicode)
        ):
            msg = "A contact email of type string is required to complete the \
                request."

            raise InvalidRequestDataException(msg)

        token, user_data, team_data = self._create_team(payload)

        endpoint = "team/apikey"
        payload = {
            "team_role": user_data["team_role"],
            "name": "{} API Key".format(team_data["company_name"]),
        }

        response = json.loads(
            self.request.post(endpoint=endpoint, payload=payload)
        )

        return (response["api_key"], Team(team_data))



from datetime import datetime


class User(object):
    """
    User model class
    """
    def __init__(self, data):
        self.id = data.get("id")
        """ Id of the user """

        self.system_admin = data.get("system_admin")
        """ Whether the user is a system admin or not """

        self.name = data.get("name")
        """ Name of the user """

        self.email = data.get("email")
        """ Email of the user """

        self.email_verification_sent_at = datetime.strptime(
            data.get("email_verification_sent_at"), '%Y-%m-%d %H:%M:%S'
        ) if data.get("email_verification_sent_at") else None
        """ When the verification email was sent """

        self.email_verified_at = datetime.strptime(
            data.get("email_verified_at"), '%Y-%m-%d %H:%M:%S'
        ) if data.get("email_verified_at") else None
        """ When the email was verified """

        self.is_api_key = data.get("is_api_key")
        """ Whether this user is an api key or not """

        self.acknowledgments = data.get("acknowledgments")
        """ Any acknowledgments for the user """

        self.created_at = datetime.strptime(
            data.get("created_at"), '%Y-%m-%d %H:%M:%S'
        ) if data.get("created_at") else None
        """ When the user was created """

        self.updated_at = datetime.strptime(
            data.get("updated_at"), '%Y-%m-%d %H:%M:%S'
        ) if data.get("updated_at") else None
        """ When the user was updated """

        self.deleted_at = datetime.strptime(
            data.get("deleted_at"), '%Y-%m-%d %H:%M:%S'
        ) if data.get("deleted_at") else None
        """ When the user was deleted """

        self.brand = data.get("brand")
        """ Brand of the user """

        self.gravatar = data.get("gravatar")
        """ Gravatar of the user """

        self.team_admin = data.get("team_admin")
        """ Whether the user is a team admin or now """

        self.team_enabled = data.get("team_enabled")
        """ Whether or not this user is enabled for a team  """

        self.team_role = data.get("team_role")
        """ The role this user has on the associated team """

        self.merchant_options = data.get("merchant_options")
        """ Settings for the user for a merchant """

        self.is_default = data.get("is_default")
        """ Default merchant to load when user logs in to Omni """
    def __repr__(self):
        repr = '{}(' \
            'id: {!r}, ' \
            'system_admin: {!r}, ' \
            'name: {!r}, ' \
            'email: {!r}, ' \
            'email_verification_sent_at: {!r}, ' \
            'email_verified_at: {!r}, ' \
            'is_api_key: {!r}, ' \
            'acknowledgments: {!r}, ' \
            'created_at: {!r}, ' \
            'updated_at: {!r}, ' \
            'deleted_at: {!r}, ' \
            'brand: {!r}, ' \
            'gravatar: {!r}, ' \
            'team_admin: {!r}, ' \
            'team_enabled: {!r}, ' \
            'team_role: {!r}, ' \
            'merchant_options: {!r}, ' \
            'is_default: {!r})'.format(
                self.__class__.__name__,
                self.id,
                self.system_admin,
                self.name,
                self.email,
                self.email_verification_sent_at,
                self.email_verified_at,
                self.is_api_key,
                self.acknowledgments,
                self.created_at,
                self.updated_at,
                self.deleted_at,
                self.brand,
                self.gravatar,
                self.team_admin,
                self.team_enabled,
                self.team_role,
                self.merchant_options,
                self.is_default
            )

        return repr



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="fattmerchant",
    version="1.0.1",
    author="Austin Burns",
    author_email="austin.burns@fattmerchant.com",
    description="Python client for Fattmerchant",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/fattmerchantorg/fattmerchant_python_client",
    packages=setuptools.find_packages(),
    package_data={
        "": ["*.config", "*.txt"],
    },
    install_requires=["ddt", "requests"],
    python_requires='>=2.7',
    classifiers=[
        "Programming Language :: Python :: 2",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



from datetime import datetime

from fattmerchant.models import Customer, Merchant, PaymentMethod, User


class Transaction(object):
    """
    Transaction model class
    """
    def __init__(self, data):
        self.id = data.get("id")
        """ Id of the transaction """

        self.invoice_id = data.get("invoice_id")
        """ Invoice id of the transaction """

        self.reference_id = data.get("reference_id")
        """ Reference id of the transaction """

        self.recurring_transaction_id = data.get("recurring_transaction_id")
        """ Recurring transaction id of the transaction """

        self.auth_id = data.get("auth_id")
        """ Auth id of the transaction """

        self.type = data.get("type")
        """ The transaction type """

        self.source = data.get("source")
        """
        Optional field to track the source of a transaction made
        outside of Omni
        """

        self.is_merchant_present = data.get("is_merchant_present")
        """
        Whether the merchant was present when the transaction was created
        """

        self.merchant_id = data.get("merchant_id")
        """ Merchant id of the transaction """

        self.user_id = data.get("user_id")
        """ User id of the transaction """

        self.customer_id = data.get("customer_id")
        """ Customer id of the transaction """

        self.payment_method_id = data.get("payment_method_id")
        """ Payment method id of the transaction """

        self.is_manual = data.get("is_manual")
        """ Whether the transaction was created manually or not """

        self.success = data.get("success")
        """ Whether the transaction was successful or not """

        self.message = data.get("message")
        """ Error message of an unsuccesful transaction """

        self.meta = data.get("meta")
        """ Any meta data of the transaction """

        self.total = data.get("total")
        """ Total of the transaction """

        self.method = data.get("method")
        """ The payment method type for the transaction """

        self.pre_auth = data.get("pre_auth")
        """ Whether the transaction is a pre authorization transaction """

        self.is_captured = data.get("is_captured")
        """ Whether the pre auth transaction is captured or not """

        self.last_four = data.get("last_four")
        """ Last for numbers of the payment method for the transaction """

        self.interchange_code = data.get("interchange_code")
        """ Interchange classification """

        self.interchange_fee = data.get("interchange_fee")
        """ Fees associated with transaction """

        self.batch_id = data.get("batch_id")
        """ (Not used) """

        self.batched_at = datetime.strptime(
            data.get("batched_at"), '%Y-%m-%d %H:%M:%S'
        ) if data.get("batched_at") else None
        """ When the transaction was batched """

        self.emv_response = data.get("emv_response")
        """ Response from card present emv transactions """

        self.avs_response = data.get("avs_response")
        """ Response from address verification system """

        self.cvv_response = data.get("cvv_response")
        """ Response from card verification """

        self.pos_entry = data.get("pos_entry")
        """ Additional details from point of sale system """

        self.pos_salesperson = data.get("pos_salesperson")
        """ Additional details from point of sale system """

        self.receipt_email_at = data.get("receipt_email_at")
        """ When the reciept email was sent """

        self.receipt_sms_at = data.get("receipt_sms_at")
        """ When the reciept text was sent """

        self.settled_at = datetime.strptime(
            data.get("settled_at"), '%Y-%m-%d %H:%M:%S'
        ) if data.get("settled_at") else None
        """ When the transaction was settled """

        self.created_at = datetime.strptime(
            data.get("created_at"), '%Y-%m-%d %H:%M:%S'
        ) if data.get("created_at") else None
        """ When the transaction was created """

        self.updated_at = datetime.strptime(
            data.get("updated_at"), '%Y-%m-%d %H:%M:%S'
        ) if data.get("updated_at") else None
        """ When the transaction was updated """

        self.source_ip = data.get("source_ip")
        """ Where the transaction was made """

        self.gateway_id = data.get("gateway_id")
        """ Gateway id of the transaction """

        self.issuer_auth_code = data.get("issuer_auth_code")
        """ Authorization code from the gateway for the transaction """

        self.total_refunded = data.get("total_refunded")
        """ Total amount that was refunded of the transaction """

        self.is_refundable = data.get("is_refundable")
        """ Whether the transaction is refundable or not """

        self.is_voided = data.get("is_voided")
        """ Whether the transaction was voided or not """

        self.is_voidable = data.get("is_voidable")
        """ Whether the transaction is voidable or not """

        self.schedule_id = data.get("schedule_id")
        """ Id of the associated schedule for this transaction  """

        self.child_captures = data.get("child_captures")
        """ Any child transaction captures if the transaction is a pre auth """

        self.parent_auth = data.get("parent_auth")
        """ The parent transaction if this transaction is a capture """

        self.gateway_name = data.get("gateway_name")
        """ Gateway name for the transaction """

        self.response = data.get("response")
        """ Optional JSON field to store the gateway response """

        self.is_settling = data.get("is_settling")
        """ Whether the transaction is settling or not """

        self.customer = Customer(data.get("customer")) \
            if data.get("customer") else None
        """ The customer for the transaction """

        child_transactions = data.get("child_transactions")
        if child_transactions:
            self.child_transactions = []

            for transaction in child_transactions:
                self.child_transactions.append(Transaction(transaction))
        else:
            self.child_transactions = None
        """ Any child transactions that are linked to the transaction """

        self.files = data.get("files")
        """ Any files that are linked to the transaction """

        self.payment_method = PaymentMethod(data.get("payment_method")) \
            if data.get("payment_method") else None
        """ Payment method for the transaction """

        self.user = User(data.get("user")) \
            if data.get("user") else None
        """ User for the transaction """

        self.merchant = Merchant(data.get("merchant")) \
            if data.get("merchant") else None
        """ Merchant for the transaction """

    def __repr__(self):
        repr = '{}(' \
            'id: {!r}, ' \
            'invoice_id: {!r}, ' \
            'reference_id: {!r}, ' \
            'recurring_transaction_id: {!r}, ' \
            'auth_id: {!r}, ' \
            'type: {!r}, ' \
            'source: {!r}, ' \
            'is_merchant_present: {!r}, ' \
            'merchant_id: {!r}, ' \
            'user_id: {!r}, ' \
            'customer_id: {!r}, ' \
            'payment_method_id: {!r}, ' \
            'is_manual: {!r}, ' \
            'success: {!r}, ' \
            'message: {!r}, ' \
            'meta: {!r}, ' \
            'total: {!r}, ' \
            'method: {!r}, ' \
            'pre_auth: {!r}, ' \
            'is_captured: {!r}, ' \
            'last_four: {!r}, ' \
            'interchange_code: {!r}, ' \
            'interchange_fee: {!r}, ' \
            'batch_id: {!r}, ' \
            'batched_at: {!r}, ' \
            'emv_response: {!r}, ' \
            'avs_response: {!r}, ' \
            'cvv_response: {!r}, ' \
            'pos_entry: {!r}, ' \
            'pos_salesperson: {!r}, ' \
            'receipt_email_at: {!r}, ' \
            'receipt_sms_at: {!r}, ' \
            'settled_at: {!r}, ' \
            'created_at: {!r}, ' \
            'updated_at: {!r}, ' \
            'source_ip: {!r}, ' \
            'gateway_id: {!r}, ' \
            'issuer_auth_code: {!r}, ' \
            'total_refunded: {!r}, ' \
            'is_refundable: {!r}, ' \
            'is_voided: {!r}, ' \
            'is_voidable: {!r}, ' \
            'schedule_id: {!r}, ' \
            'child_captures: {!r}, ' \
            'parent_auth: {!r}, ' \
            'gateway_name: {!r}, ' \
            'response: {!r}, ' \
            'is_settling: {!r}, ' \
            'customer: {!r}, ' \
            'child_transactions: {!r}, ' \
            'files: {!r}, ' \
            'payment_method: {!r}, ' \
            'user: {!r}, ' \
            'merchant: {!r})'.format(
                self.__class__.__name__,
                self.id,
                self.invoice_id,
                self.reference_id,
                self.recurring_transaction_id,
                self.auth_id,
                self.type,
                self.source,
                self.is_merchant_present,
                self.merchant_id,
                self.user_id,
                self.customer_id,
                self.payment_method_id,
                self.is_manual,
                self.success,
                self.message,
                self.meta,
                self.total,
                self.method,
                self.pre_auth,
                self.is_captured,
                self.last_four,
                self.interchange_code,
                self.interchange_fee,
                self.batch_id,
                self.batched_at,
                self.emv_response,
                self.avs_response,
                self.cvv_response,
                self.pos_entry,
                self.pos_salesperson,
                self.receipt_email_at,
                self.receipt_sms_at,
                self.settled_at,
                self.created_at,
                self.updated_at,
                self.source_ip,
                self.gateway_id,
                self.issuer_auth_code,
                self.total_refunded,
                self.is_refundable,
                self.is_voided,
                self.is_voidable,
                self.schedule_id,
                self.child_captures,
                self.parent_auth,
                self.gateway_name,
                self.response,
                self.is_settling,
                self.customer,
                self.child_transactions,
                self.files,
                self.payment_method,
                self.user,
                self.merchant
            )

        return repr



import json
import logging

from fattmerchant.exceptions import InvalidRequestDataException
from fattmerchant.models import DepositBatch, DepositDetails

logger = logging.getLogger(__name__)


class DepositsController(object):
    """
    Class to interface with deposits within the Fattmerchant API
    """
    def __init__(self, request):
        self.request = request
        self.api = "fq"

    def list(self, options):
        """
        Gets a list of deposit batches within a certain date range from the API

        :param options: A dictionary with any optional query params

            .. note:: **start_date** or **end_date** is required
            .. code-block:: json

                {
                    "start_date": "1970-01-01",
                    "end_date": "1970-01-01",
                    "timezone": "EST",
                    "timezoneOffset": 0
                }

        :type options: dict

        :return: A list of deposit batch objects
        :rtype: :doc:`../models/deposit_batch`

        :raise InvalidRequestDataException: Raised if **start_date** is not
            provided or **end_date** is not provided

        """

        # Check if any of the dates are part of the options
        if ("start_date" not in options and "end_date" not in options):
            # yapf: enable
            msg = "At least a start_date or end_date is required to " \
                "make this request."

            raise InvalidRequestDataException(msg)

        # Make sure that the start date is of either type string or unicode
        if (
            "start_date" in options
            and not isinstance(options["start_date"], (str, unicode))
        ):
            msg = "The start date needs to be of type string."

            raise InvalidRequestDataException(msg)
        else:
            # Change the date keys to keys the API accepts
            options["startDate"] = options.pop("start_date")

        # Make sure that the start date is of either type string or unicode
        if (
            "end_date" in options
            and not isinstance(options["end_date"], (str, unicode))
        ):
            msg = "The end date needs to be of type string."

            raise InvalidRequestDataException(msg)
        else:
            # Change the date keys to keys the API accepts
            options["endDate"] = options.pop("end_date")

        endpoint = "deposit"
        response = json.loads(self.request.get(self.api, endpoint, options))

        deposits = []

        for deposit_data in response["data"]:
            deposits.append(DepositBatch(deposit_data))

        return deposits

    def get(self, options):
        """
        Gets a list of deposit batch details within a certain date range from \
        the API

        :param options: A dictionary with any optional query params

            .. note:: **start_date** or **end_date** is required
            .. code-block:: json

                {
                    "start_date": "1970-01-01",
                    "end_date": "1970-01-01",
                    "timezone": "EST",
                    "timezoneOffset": 0
                }

        :type options: dict

        :return: A list of deposit details objects
        :rtype: :doc:`../models/deposit_details`

        :raise InvalidRequestDataException: Raised if **start_date** is not
            provided or **end_date** is not provided

        """

        # Check if any of the dates are part of the options
        if ("start_date" not in options and "end_date" not in options):
            # yapf: enable
            msg = "At least a start_date or end_date is required to " \
                "make this request."

            raise InvalidRequestDataException(msg)

        # Make sure that the start date is of either type string or unicode
        if ("start_date" in options):
            if (not isinstance(options["start_date"], (str, unicode))):
                msg = "The start date needs to be of type string. "

                raise InvalidRequestDataException(msg)
            else:
                # Change the date keys to keys the API accepts
                options["startDate"] = options.pop("start_date")

        # Make sure that the end date is of either type string or unicode
        if ("end_date" in options):
            if (not isinstance(options["end_date"], (str, unicode))):
                msg = "The end date needs to be of type string."

                raise InvalidRequestDataException(msg)
            else:
                # Change the date keys to keys the API accepts
                options["endDate"] = options.pop("end_date")

        endpoint = "depositDetail"
        response = json.loads(self.request.get(self.api, endpoint, options))

        deposits = []

        for deposit_data in response["data"]:
            deposits.append(DepositDetails(deposit_data))

        return deposits
